var DataV = require('dchart-core/datav');
var d3 = require('d3');
var _ = require('dchart-core/util');

/**
 * @param container
 * @param options
 * @constructor
 */
function Pie(container, options) {
  var opt = {
    arc: {
      startAngle : function (d) {return d.startAngle; },
      endAngle : function (d) {return d.endAngle;},
      net: false,
      netColor: '#aaa',
    },
    pyaxis: {
      key: 'y'
    },
    paxis : null,
    "default-color": "#1F6086",
    colors: []
  };
  opt = _.deepMerge(opt, options);
  DataV.call(this, container, opt);
}

Pie = DataV.extend(Pie, {
  beforeRender : function () {
    var opt = this.options;
    if (!opt.arc.radius)  {opt.arc._radius = Math.min(opt.innerHeight, opt.innerWidth) / 2;}
    else {opt.arc._radius = opt.arc.radius;}

    var self = this;
    this.dealColor(opt, 'default-color');
    opt.colors && opt.colors.length && opt.colors.forEach(function (serie, i) {
      self.dealColor(opt.colors, '[' + i + ']')
    })
  },
  updateBeforeRender : function () {
    this.beforeRender();
  },
  changeSeries : function () {
    var opt = this.options;
    var data = this.data();
    var pie = d3.layout.pie()
      .sort(null)
      .value(function(d) {
        var value = d[opt.pyaxis.key];
        if (typeof value === 'string') {
          return _.toNumber(value)
        } else if (!value) {
          return 0;
        }
        return value;
      });

    var arc = this.getComs('arc', 'arc').getArc();
    var pieData = pie(data);

    var pies = this.pies.attr("transform", "translate(" + opt.innerWidth / 2 + "," + opt.innerHeight / 2 + ")")
      .selectAll('.serie')
      .data(pieData)
    var labels = this.labels.attr("transform", "translate(" + opt.innerWidth / 2 + "," + opt.innerHeight / 2 + ")")
      .selectAll('.serie-label')
      .data(pieData)

    var trans = function (d) {
      if (!opt.paxis._dr) {
        return "translate(" + arc.centroid(d) + ")";
      }
      var dr = opt.paxis._dr;
      var angle = (d.startAngle + d.endAngle) / 2;
      return "translate(" + (dr * Math.sin(angle)) + ", " + (-dr * Math.cos(angle)) + ")";
    }

    var html = function(d, i) {
      if (typeof opt.paxis.label === 'function') {
        return opt.paxis.label.call(d, d.data, i);
      }
      if (opt.paxis.label && opt.paxis.label.length) {
        return opt.paxis.label[i];
      }
      return d.data[opt.paxis.key];
    };
    this.pies.selectAll('.serie').each(function (d, i) {
      var serie = d3.select(this);
      if (opt.animation) {
        serie.transition()
          .duration(opt.animationDuration)
          .attrTween("d", function() {
            this._cur = this._cur || d;
            var interpolate = d3.interpolate(this._cur, d);
            this._cur = d;
            return function(t) {
              return arc(interpolate(t), i);
            };
          })
      } else {
        serie.attr({d: function () {return arc(d, i);}})
      }
    });

    this.labels.selectAll('.serie-label').each(function (d, i) {
      var label = d3.select(this);
      if (opt.aniamtion) {
        if (opt.paxis) {
          label.html(html.bind(null, d, i)).transition()
            .duration(opt.animationDuration)
            .attrTween("transform", function() {
              this._cur = this._cur || d;
              var interpolate = d3.interpolate(this._cur, d);
              this._cur = d;
              return function(t) {
                return trans(interpolate(t))
              };
            })
        }
      } else {
        if (opt.paxis) {
          label.attr("transform", trans.bind(null, d)).html(html.bind(null, d, i));
        }
      }
    });

    pies
      .enter()
      .append('path')
      .each(function (d, i) {
        var that = d3.select(this);
        if (opt.animation) {
          that.attr({
              "class": function () {return "serie serie1";},
            }).transition()
            .duration(opt.animationDuration)
            .attrTween("d", function() {
              this._cur = d;
              var interpolate = d3.interpolate({endAngle: d.startAngle}, d);
              return function(t) {
                return arc(interpolate(t), i);
              };
            })

        } else {
          that.attr({
              "class": function (d, j) {return "serie serie1";},
              "d": function () {return arc(d, i);}
            });
        }
      })

    labels
      .enter()
      .append('text')
      .attr({
        "class": "serie-label",
        "transform": trans,
        "dy": ".35em"
      }).style("text-anchor", "middle")
      .html(html).each(function (d) {
      this._cur = d;
    })

    pies.exit().remove();
    labels.exit().remove();

    var net = this.svg.select('.net-g');
    if (opt.arc.net) {
      if (!net[0][0]) {
        net = this.svg.insert('g', '.series')
          .attr('class', 'net-g');
      }
      var netData = this._calNet(opt.arc._radius);
      var nets = net.style('display', 'block')
        .attr('transform', 'translate(' + opt.innerWidth / 2 + "," + opt.innerHeight / 2 + ')')
        .selectAll('.net-path')
        .data(netData)

      nets.enter()
        .append('circle')
        .attr({
          'class': 'net-path',
          cx: 0,
          cy: 0
        })

      nets.exit().remove();

      nets.attr({r: function(d) {return d;}}).style({
        stroke: opt.arc.netColor,
        fill: 'none',
        'stroke-dasharray': '4 4'
      })
    } else if (net[0][0]) {
      net.style('display', 'none');
    }
  },
  renderSeries : function () {
    this.svg.classed('pie-chart-g', true);
    var opt = this.options;
    var arc = this.renderCom('arc', 'arc', this.svg[0][0], opt.arc).getArc();
    this.pies = this.series.append('g').attr('class', 'series-pies')
    this.labels = this.series.append('g').attr('class', 'series-labels')
    this.changeSeries();
  },
  updateSeries : function () {
    var opt = this.options;
    var data = this.data();
    var arc = this.getComs('arc', 'arc')
    arc.update(data, opt.arc);

    this.changeSeries();
  },
  afterRender: function () {
    var opt = this.options;
    this.series.selectAll('.series-group .serie').style({
      fill: opt['default-color'].res
    })
    if (opt.colors.length) {
      for (var i = 0; i < opt.colors.length; i++) {
        this.series.selectAll('.series-group:nth-child(' + (i+1) + ') .serie').style({
          fill: opt.colors[i].res || opt['default-color'].res
        })
      }
    }
  },
  updateAfterRender: function () {
    Pie.prototype.afterRender.call(this);
  }
});

module.exports = Pie;
